/*
 * SpeechPlayer.mainForm
 * author: Makoto Yoshida
 * version: 2008/6/30
 * Microsoft Windows 98 or later
 * Microsoft .NET Framework 2.0 or later
 * Microsoft Speech Object Library 5.1 or later
 */

using SpeechLib;
using System;
using System.Reflection;
using System.Windows.Forms;

namespace SpeechPlayer
{
    public partial class mainForm : Form
    {
        private const string lastSentence = ".  The end.";

        internal SpVoiceClass theSpVoice = new SpVoiceClass(); // SpVoice or SpVoiceClass
        private int currentStreamNumber;
        private string currentText = string.Empty;
        private bool loopLooping = false;
        private int loopPivot = 0;
        private SpeechPlayer.settingsForm theSettingsForm = new SpeechPlayer.settingsForm();

        public mainForm()
        {
            this.InitializeComponent();
            this.theTableLayoutPanel.Margin = new Padding(0, 0, 0, this.theStatusStrip.Height);

            theSpVoice.Word
                += new _ISpeechVoiceEvents_WordEventHandler(theSpVoice_Word);
            theSpVoice.Sentence
                += new _ISpeechVoiceEvents_SentenceEventHandler(theSpVoice_Sentence);
            theSpVoice.EndStream
                += new _ISpeechVoiceEvents_EndStreamEventHandler(theSpVoice_EndStream);
            setVoice(SpeechPlayer.Properties.Settings.Default.Voice);
            theSpVoice.SetRate((int)SpeechPlayer.Properties.Settings.Default.Rate);
            theSpVoice.SetVolume((ushort)SpeechPlayer.Properties.Settings.Default.Volume);
        }

        internal void setVoice(string Name)
        {
            ISpeechObjectTokens sot = theSpVoice.GetVoices(string.Empty, string.Empty);
            theSpVoice.Voice = sot.Item(0); // default
            SpeechPlayer.Properties.Settings.Default.Voice =
                sot.Item(0).GetDescription(0); // default
            for (int i = 0; i < sot.Count; i++)
            {
                if (string.Compare(sot.Item(i).GetDescription(0), Name) == 0)
                {
                    theSpVoice.Voice = sot.Item(i);
                    SpeechPlayer.Properties.Settings.Default.Voice = Name;
                }
            }
        }

        private void beginSpeech(string Text)
        {
            if (this.pauseCheckBox.Checked) this.pauseCheckBox.Checked = false;
            theSpVoice.Resume();
            currentText = Text;
            currentText = currentText.Replace("\r", " ");
            currentText = currentText.Replace("\n", " ");
            try
            {
                currentStreamNumber = theSpVoice.Speak(currentText + lastSentence,
                    SpeechVoiceSpeakFlags.SVSFlagsAsync |
                    SpeechVoiceSpeakFlags.SVSFPurgeBeforeSpeak);
            }
            catch (System.Runtime.InteropServices.COMException e)
            {
                System.Diagnostics.Debug.WriteLine(e.StackTrace);
            }
            this.theProgressBar.Minimum = 0;
            this.theProgressBar.Maximum = currentText.Length;
            this.theProgressBar.Value = 0;
            this.theToolTip.SetToolTip(this.theStatusStrip, currentText);
        }

        private void theSpVoice_Word(int StreamNumber, object StreamPosition,
            int CharacterPosition, int Length)
        {
            if (StreamNumber != currentStreamNumber) return;
            if (currentText.Length < CharacterPosition) theSpVoice.Skip("SENTENCE", 1);
            this.theProgressBar.Value = CharacterPosition;
            this.theToolStripStatusLabel.Text = currentText.Substring(CharacterPosition);
            if (loopLooping && (loopPivot == 0))
            {
                loopPivot = CharacterPosition;
                theSpVoice.Skip("SENTENCE", 0);
            }
        }

        private void theSpVoice_Sentence(int StreamNumber, object StreamPosition,
            int CharacterPosition, int Length)
        {
            if (StreamNumber != currentStreamNumber) return;
            if (this.pauseCheckBox.Checked) return; // To avoid some unfavorable behavior.
            if (!this.backButton.Enabled) this.backButton.Enabled = true;
            if (!this.forwardButton.Enabled) this.forwardButton.Enabled = true;
            if (loopLooping && (loopPivot != 0) && (loopPivot <= CharacterPosition))
            {
                theSpVoice.Skip("SENTENCE", -1);
            }
        }

        private void theSpVoice_EndStream(int StreamNumber, object StreamPosition)
        {
            if (StreamNumber != currentStreamNumber) return;
            this.theProgressBar.Value = 0;
            this.theToolStripStatusLabel.Text = string.Empty;
            this.theToolTip.SetToolTip(this.theStatusStrip, string.Empty);
        }

        private void mainForm_DragEnter(object sender, DragEventArgs e)
        {
            // Drag and Drop does not work properly from some application programs.
            if (e.Data.GetDataPresent(DataFormats.Text, true))
            {
                e.Effect = DragDropEffects.Link;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void mainForm_DragDrop(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text, true))
            {
                beginSpeech((string)e.Data.GetData(DataFormats.Text, true));
            }
        }

        private void mainForm_Activated(object sender, EventArgs e)
        {
            this.Opacity = 1D;
        }

        private void mainForm_Deactivate(object sender, EventArgs e)
        {
            if (this.TopMost) this.Opacity = 0.8D;
        }

        private void backButton_Click(object sender, EventArgs e)
        {
            if (this.theSpVoice.Status.RunningState == SpeechRunState.SRSEDone) return;
            // To avoid conflicts between Skip methods.
            // Immediately Enabled again in the Sentence event.
            this.backButton.Enabled = false;
            this.forwardButton.Enabled = false;
            theSpVoice.Skip("SENTENCE", -1);
        }

        private void forwardButton_Click(object sender, EventArgs e)
        {
            if (this.theSpVoice.Status.RunningState == SpeechRunState.SRSEDone) return;
            // To avoid conflicts between Skip methods.
            // Immediately Enabled again in the Sentence event.
            this.backButton.Enabled = false;
            this.forwardButton.Enabled = false;
            theSpVoice.Skip("SENTENCE", 1);
        }

        private void loopLabel_MouseEnter(object sender, EventArgs e)
        {
            loopLooping = true;
            this.loopLabel.BorderStyle = BorderStyle.FixedSingle;
        }

        private void loopLabel_MouseLeave(object sender, EventArgs e)
        {
            loopLooping = false;
            loopPivot = 0;
            this.loopLabel.BorderStyle = BorderStyle.Fixed3D;
        }

        private void pauseCheckBox_MouseEnter(object sender, EventArgs e)
        {
            if (!this.pauseCheckBox.Checked) theSpVoice.Pause();
        }

        private void pauseCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            bool b = this.pauseCheckBox.Checked;
            this.backButton.Enabled = !b;
            this.forwardButton.Enabled = !b;
            this.loopLabel.Enabled = !b;
        }

        private void pauseCheckBox_MouseLeave(object sender, EventArgs e)
        {
            if (!this.pauseCheckBox.Checked) theSpVoice.Resume();
        }

        private void clipboardButton_Click(object sender, EventArgs e)
        {
            IDataObject d = Clipboard.GetDataObject();
            if (d.GetDataPresent(DataFormats.Text, true))
            {
                beginSpeech((string)d.GetData(DataFormats.Text, true));
            }
        }

        private void replayButton_Click(object sender, EventArgs e)
        {
            beginSpeech(currentText);
        }

        private void settingsButton_Click(object sender, EventArgs e)
        {
            if (!theSettingsForm.Visible) theSettingsForm.ShowDialog(this);
        }

        private void aboutButton_Click(object sender, EventArgs e)
        {
            MessageBox.Show(aboutProduct() + " version " + aboutVersion() + "\n" +
                aboutCopyright(), aboutTitle());
        }

        private string aboutTitle()
        {
            object[] o = Assembly.GetExecutingAssembly().GetCustomAttributes(
                typeof(AssemblyTitleAttribute), false);
            if (o.Length != 0)
            {
                return ((AssemblyTitleAttribute)o[0]).Title;
            }
            else
            {
                return string.Empty;
            }
        }

        private string aboutProduct()
        {
            object[] o = Assembly.GetExecutingAssembly().GetCustomAttributes(
                typeof(AssemblyProductAttribute), false);
            if (o.Length != 0)
            {
                return ((AssemblyProductAttribute)o[0]).Product;
            }
            else
            {
                return string.Empty;
            }
        }

        private string aboutVersion()
        {
            return Assembly.GetExecutingAssembly().GetName().Version.ToString();
        }

        private string aboutCopyright()
        {
            object[] o = Assembly.GetExecutingAssembly().GetCustomAttributes(
                typeof(AssemblyCopyrightAttribute), false);
            if (o.Length != 0)
            {
                return ((AssemblyCopyrightAttribute)o[0]).Copyright;
            }
            else
            {
                return string.Empty;
            }
        }
    }
}